//
//  ExcelTable.swift
//  AutoSMS
//
//  Created by Mark Johnson on 2/24/20.
//  Copyright © 2020 matzsoft. All rights reserved.
//

import Foundation
import CoreXLSX

extension Worksheet {
    var minColumn: Int {
        return ( self.data?.rows ?? [] ).reduce( Int.max, {
            min( $0, $1.cells.first?.reference.column.intValue ?? Int.max )
        } )
    }
    var maxColumn: Int {
        return ( self.data?.rows ?? [] ).reduce( 0, {
            max( $0, $1.cells.last?.reference.column.intValue ?? 0 )
        } )
    }
}

func ExcelTable( filepath: String ) -> [[String]] {
    guard let file = XLSXFile( filepath: filepath ) else {
        fatalError( "XLSX file \(filepath) corrupted or does not exist" )
    }
    
    var rows: [[String]] = []
    
    do {
        let wsPaths = try file.parseWorksheetPaths()
        let sharedStrings = try file.parseSharedStrings()
        let ws = try file.parseWorksheet( at: wsPaths[0] )
        
        for wsRow in ws.data?.rows ?? [] {
            var row: [String] = []
            var nextColumn = ws.minColumn

            for cell in wsRow.cells {
                let thisColumn = cell.reference.column.intValue
                
                row.append( contentsOf: Array( repeating: "", count: thisColumn - nextColumn ) )
                nextColumn = thisColumn + 1

                guard let value = cell.value else {
                    row.append( "" )
                    continue
                }
                
                guard let type = cell.type else {
                    row.append( value.trimmingCharacters( in: CharacterSet( charactersIn: " " ) ) )
                    continue
                }
                
                if type == .sharedString {
                    let shared = sharedStrings.items[ Int( cell.value! )! ].text!
                    row.append( shared.trimmingCharacters( in: CharacterSet( charactersIn: " " ) ) )
                } else {
                    fatalError( "Unknow cell type \(type) at \(cell.reference)" )
                }
            }
            row.append( contentsOf: Array( repeating: "", count: ws.maxColumn + 1 - nextColumn ) )
            if !row.allSatisfy( { $0 == "" } ) {
                rows.append( row )
            }
        }
    } catch {
        fatalError( "Error parsing XLSX file \(filepath )" )
    }
    
    return rows
}
